@@ -1,6 +1,6 @@
 /**
  * OpenGL ES 2.0 memory performance estimator
- * Copyright (C) 2009 Nokia
+ * Copyright (C) 2010 Nokia
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,8 +20,14 @@
  */
 #include <GLES2/gl2.h>
 #include <GLES2/gl2ext.h>
+
 #include <X11/Xutil.h>
+
 #include <EGL/egl.h>
+
+#include <sys/stat.h>
+
+#include <stdio.h>
 #include <string>
 #include <unistd.h>
 #include <time.h>
@@ -29,7 +35,6 @@
 #include <list>
 #include <iostream>
 #include <algorithm>
-#include <sys/stat.h>
 
 #include "native.h"
 #include "util.h"
@@ -41,6 +46,7 @@
 #include "shaderblittest.h"
 #include "cpuinterleavingtest.h"
 #include "config.h"
+#include "ext.h"
 
 #if defined(HAVE_LIBOSSO)
 #include <libosso.h>
@@ -54,6 +60,7 @@ static struct
     int                    bitsPerPixel;
     bool                   verbose;
     int                    minTime;
+    bool                   listTests;
     std::list<std::string> includedTests;
     std::list<std::string> excludedTests;
 } options;
@@ -68,13 +75,12 @@ osso_context_t* ossoContext;
 bool initializeEgl(int width, int height, const EGLint* configAttrs, const EGLint* contextAttrs)
 {
     EGLint configCount = 0;
-    
+
 #if defined(HAVE_LIBOSSO)
     ossoContext = osso_initialize("com.nokia.memperf", "1.0", FALSE, NULL);
     if (!ossoContext)
     {
-        perror("osso_initialize");
-        goto out_error;
+        printf("Warning: osso_initialize failed\n");
     }
 #endif
 
@@ -111,7 +117,7 @@ bool initializeEgl(int width, int height, const EGLint* configAttrs, const EGLin
         printf("Unable to create a context\n");
         goto out_error;
     }
-    
+
     ctx.surface = eglCreateWindowSurface(ctx.dpy, ctx.config, ctx.win, NULL);
     ASSERT_EGL();
     if (!ctx.surface)
@@ -122,7 +128,7 @@ bool initializeEgl(int width, int height, const EGLint* configAttrs, const EGLin
 
     eglMakeCurrent(ctx.dpy, ctx.surface, ctx.surface, ctx.context);
     ASSERT_EGL();
-    
+
     eglSwapInterval(ctx.dpy, 0);
     return true;
 
@@ -146,8 +152,11 @@ void terminateEgl()
     nativeDestroyDisplay(ctx.nativeDisplay);
 
 #if defined(HAVE_LIBOSSO)
-    osso_deinitialize(ossoContext);
-    ossoContext = 0;
+    if (ossoContext)
+    {
+        osso_deinitialize(ossoContext);
+        ossoContext = 0;
+    }
 #endif
 }
 
@@ -196,6 +205,12 @@ void runTest(Test& test)
     int64_t minTime = options.minTime * 1000 * 1000 * 1000LL;
     struct timespec res, start, end;
 
+    if (options.listTests)
+    {
+        printf("%s\n", test.name().c_str());
+        return;
+    }
+
     if (!shouldRunTest(test.name()))
     {
         return;
@@ -217,6 +232,8 @@ void runTest(Test& test)
         return;
     }
 
+    nativeVerifyWindow(ctx.nativeDisplay, ctx.win);
+
     while (warmup--)
     {
         test(0);
@@ -227,7 +244,10 @@ void runTest(Test& test)
     ASSERT_EGL();
 
 #if defined(HAVE_LIBOSSO)
-    osso_display_blanking_pause(ossoContext);
+    if (ossoContext)
+    {
+        osso_display_blanking_pause(ossoContext);
+    }
 #endif
 
     clock_gettime(CLOCK_REALTIME, &start);
@@ -245,7 +265,7 @@ void runTest(Test& test)
 
     ASSERT_GL();
     ASSERT_EGL();
-    
+
     test.teardown();
     ASSERT_GL();
     ASSERT_EGL();
@@ -276,9 +296,9 @@ void getScreenSize(int& width, int& height)
 
 void showIntro()
 {
-    std::cout << 
+    std::cout <<
         "GLMemPerf v" PACKAGE_VERSION " - OpenGL ES 2.0 memory performance benchmark\n"
-        "Copyright (C) 2009 Nokia Corporation. GLMemPerf comes with ABSOLUTELY\n"
+        "Copyright (C) 2010 Nokia Corporation. GLMemPerf comes with ABSOLUTELY\n"
         "NO WARRANTY; This is free software, and you are welcome to redistribute\n"
         "it under certain conditions.\n"
         "\n";
@@ -286,12 +306,13 @@ void showIntro()
 
 void showUsage()
 {
-    std::cout << 
+    std::cout <<
         "Usage:\n"
         "       glmemperf [OPTIONS]\n"
         "Options:\n"
         "       -h             This text\n"
         "       -v             Verbose mode\n"
+        "       -l             List all tests without running them\n"
         "       -i TEST        Include a specific test (full name or substring)\n"
         "       -e TEST        Exclude a specific test (full name or substring)\n"
         "       -t SECS        Minimum time to run each test\n"
@@ -306,6 +327,7 @@ void parseArguments(const std::list<std::string>& args)
     options.verbose = false;
     options.minTime = 1;
     options.bitsPerPixel = 16;
+    options.listTests = false;
 
     for (i = args.begin(), i++; i != args.end(); ++i)
     {
@@ -334,6 +356,10 @@ void parseArguments(const std::list<std::string>& args)
         {
             options.verbose = true;
         }
+        else if (*i == "-l")
+        {
+            options.listTests = true;
+        }
         else
         {
             std::cerr << "Invalid option: " << *i << std::endl;
@@ -351,7 +377,7 @@ void findDataDirectory()
         return;
     }
     chdir(PREFIX "/share/glmemperf/");
-    assert(stat("data", &st) == 0);
+    ASSERT(stat("data", &st) == 0);
 }
 
 #define ADD_TEST(TEST) runTest(*std::auto_ptr<Test>(new TEST));
@@ -389,11 +415,11 @@ int main(int argc, char** argv)
     EGLint configCount = 0;
 
     bool result = nativeCreateDisplay(&ctx.nativeDisplay);
-    assert(result);
+    ASSERT(result);
 
     getScreenSize(winWidth, winHeight);
     result = initializeEgl(winWidth, winHeight, configAttrs, contextAttrs);
-    assert(result);
+    ASSERT(result);
 
     eglChooseConfig(ctx.dpy, configAttrs32, &config32, 1, &configCount);
 
@@ -411,12 +437,16 @@ int main(int argc, char** argv)
     // Normal blits
     ADD_TEST(BlitTest(GL_RGBA, GL_UNSIGNED_BYTE,              800, 480, "data/water2_800x480_rgba8888.raw"));
     ADD_TEST(BlitTest(GL_RGB,  GL_UNSIGNED_BYTE,              800, 480, "data/water2_800x480_rgb888.raw"));
+    ADD_TEST(BlitTest(GL_RGBA, GL_UNSIGNED_BYTE,              864, 480, "data/water2_864x480_rgba8888.raw"));
+    ADD_TEST(BlitTest(GL_RGB,  GL_UNSIGNED_BYTE,              864, 480, "data/water2_864x480_rgb888.raw"));
     ADD_TEST(BlitTest(GL_RGBA, GL_UNSIGNED_BYTE,             1024, 512, "data/digital_nature2_1024x512_rgba8888.raw", false, 800.0 / 1024, 480.0 / 512));
     ADD_TEST(BlitTest(GL_RGB,  GL_UNSIGNED_SHORT_5_6_5,       800, 480, "data/water2_800x480_rgb565.raw"));
     ADD_TEST(BlitTest(GL_RGB,  GL_UNSIGNED_SHORT_5_6_5,      1024, 512, "data/digital_nature2_1024x512_rgb565.raw", false, 800.0 / 1024, 480.0 / 512));
     ADD_TEST(BlitTest(GL_COMPRESSED_RGB_PVRTC_4BPPV1_IMG, 0, 1024, 512, "data/abstract3_1024x512_pvrtc4.raw", false, 800.0 / 1024, 480.0 / 512));
     ADD_TEST(BlitTest(GL_COMPRESSED_RGB_PVRTC_2BPPV1_IMG, 0, 1024, 512, "data/abstract3_1024x512_pvrtc2.raw", false, 800.0 / 1024, 480.0 / 512));
     ADD_TEST(BlitTest(GL_ETC1_RGB8_OES,                   0, 1024, 512, "data/abstract3_1024x512_etc1.raw", false, 800.0 / 1024, 480.0 / 512));
+    ADD_TEST(BlitTest(GL_LUMINANCE, GL_UNSIGNED_BYTE,         800, 480, "data/abstract3_04_800x480_r8.raw"));
+    ADD_TEST(BlitTest(GL_LUMINANCE, GL_UNSIGNED_BYTE,        1024, 512, "data/abstract3_04_1024x512_r8.raw", false, 800.0 / 1024, 480.0 / 512));
     ADD_TEST(PixmapBlitTest(w, h, ctx.config));
     ADD_TEST(PixmapBlitTest(w, h, config32));
     ADD_TEST(FBOBlitTest(GL_RGBA, GL_UNSIGNED_BYTE,          w, h));
@@ -427,6 +457,8 @@ int main(int argc, char** argv)
     // Rotated blits
     ADD_TEST(BlitTest(GL_RGBA, GL_UNSIGNED_BYTE,              480,  800, "data/water2_480x800_rgba8888.raw", true));
     ADD_TEST(BlitTest(GL_RGB,  GL_UNSIGNED_BYTE,              480,  800, "data/water2_480x800_rgb888.raw", true));
+    ADD_TEST(BlitTest(GL_RGBA, GL_UNSIGNED_BYTE,              480,  864, "data/water2_480x864_rgba8888.raw", true));
+    ADD_TEST(BlitTest(GL_RGB,  GL_UNSIGNED_BYTE,              480,  864, "data/water2_480x864_rgb888.raw", true));
     ADD_TEST(BlitTest(GL_RGBA, GL_UNSIGNED_BYTE,              512, 1024, "data/digital_nature2_512x1024_rgba8888.raw", true, 480.0 / 512, 800.0 / 1024));
     ADD_TEST(BlitTest(GL_RGB,  GL_UNSIGNED_SHORT_5_6_5,       480,  800, "data/water2_480x800_rgb565.raw", true));
     ADD_TEST(BlitTest(GL_RGB,  GL_UNSIGNED_SHORT_5_6_5,       512, 1024, "data/digital_nature2_512x1024_rgb565.raw", true, 480.0 / 512, 800.0 / 1024));
@@ -475,12 +507,16 @@ int main(int argc, char** argv)
     ADD_TEST(ShaderBlitTest("lingrad", w, h));
     ADD_TEST(ShaderBlitTest("radgrad", w, h));
     ADD_TEST(ShaderBlitTest("palette", w, h));
-    
+    ADD_TEST(ShaderBlitTest("blur", w, h));
+
     // CPU interleaving
     ADD_TEST(CPUInterleavingTest(CPUI_XSHM_IMAGE, 2, 16, winWidth, winHeight));
     ADD_TEST(CPUInterleavingTest(CPUI_XSHM_IMAGE, 2, 32, winWidth, winHeight));
     ADD_TEST(CPUInterleavingTest(CPUI_TEXTURE_UPLOAD, 2, 16, winWidth, winHeight));
     ADD_TEST(CPUInterleavingTest(CPUI_TEXTURE_UPLOAD, 2, 32, winWidth, winHeight));
 
+    ADD_TEST(CPUInterleavingTest(CPUI_EGL_LOCK_SURFACE, 2, 16, winWidth, winHeight));
+    ADD_TEST(CPUInterleavingTest(CPUI_EGL_LOCK_SURFACE, 2, 32, winWidth, winHeight));
+
     terminateEgl();
 }
